from django.db import models
from django.db.models.fields.related import SingleRelatedObjectDescriptor 
from django.core.serializers.json import DjangoJSONEncoder
from django.conf import settings
from django.forms.widgets import Textarea
from django.utils import simplejson


class AutoSingleRelatedObjectDescriptor(SingleRelatedObjectDescriptor):
    def __get__(self, instance, instance_type=None):
        try:
            return super(AutoSingleRelatedObjectDescriptor, self).__get__(instance, instance_type)
        except self.related.model.DoesNotExist:
            obj = self.related.model(**{self.related.field.name: instance})
            obj.save()
            return obj


class AutoOneToOneField(models.OneToOneField):
    """
    OneToOneField creates dependent object on first request from parent object
    if dependent oject has not created yet.

    Origin: http://softwaremaniacs.org/blog/2007/03/07/auto-one-to-one-field/
    """

    def contribute_to_related_class(self, cls, related):
        setattr(cls, related.get_accessor_name(), AutoSingleRelatedObjectDescriptor(related))




class JSONField(models.TextField):
    __metaclass__ = models.SubfieldBase

    def contribute_to_class(self, cls, name):
        super(JSONField, self).contribute_to_class(cls, name)

        def get_json(model):
            return self.get_db_prep_value(getattr(model, self.attname))
        setattr(cls, 'get_%s_json' % self.name, get_json)

        def set_json(model, json):
            setattr(model, self.attname, self.to_python(json))
        setattr(cls, 'set_%s_json' % self.name, set_json)

    def formfield(self, **kwargs):
        kwargs['widget'] = JSONWidget(attrs={'class': 'vLargeTextField'})
        return super(JSONField, self).formfield(**kwargs)

    def get_db_prep_value(self, value):
        return simplejson.dumps(value)

    def to_python(self, value):
        if not isinstance(value, basestring):
            return value

        try:
            return simplejson.loads(value, encoding=settings.DEFAULT_CHARSET)
        except ValueError, e:
            # If string could not parse as JSON it's means that it's Python
            # string saved to JSONField.
            return value


class JSONWidget(Textarea):
    """
    Prettify dumps of all non-string JSON data.
    """
    def render(self, name, value, attrs=None):
        if not isinstance(value, basestring) and value is not None:
            value = simplejson.dumps(value, indent=4, sort_keys=True)
        return super(JSONWidget, self).render(name, value, attrs)



# -*- coding: utf-8
from cgi import parse_qsl
from urllib import urlencode

from django import template
from django.utils.encoding import smart_str


register = template.Library()

@register.simple_tag
def alter_qs(qs, name, value, name2=None, value2=None):
    """
    Alter query string argument with new value.
    """

    qs = qs.lstrip('?')
    args = [[x[0], smart_str(x[1])] for x in parse_qsl(qs)]
    if value:
        found = False
        for arg in args:
            if arg[0] == name:
                arg[1] = smart_str(value)
                found = True
        if not found:
            args.append([name, smart_str(value)])
    else:
        args = [x for x in args if x[0] != name]
    if args:
        args = [tuple(x) for x in args]
        result = '?' + urlencode(args)
        if name2 and value2:
            return alter_qs(result, name2, value2)
        else:
            return result
    else:
        return ''



# -*- coding: utf-8 -*-
from django.utils import simplejson
from django.template.loader import render_to_string
from django.http import HttpResponseRedirect, HttpResponse
from django.conf import settings

class HttpResponseJson(HttpResponse):
    def __init__(self, data):
        json_data = simplejson.dumps(data)
        super(HttpResponseJson, self).__init__(
            content=json_data, mimetype='application/json')


def backurl(request, default='/'):
    """
    Return previous url from user's browser history using Referer
    header or ``default`` as fallback value.
    """

    return request.META.get('HTTP_REFERER', None) or default



# -*- coding: utf-8 -*-
import traceback

from django.shortcuts import render_to_response
from django.template import RequestContext

from common.http import HttpResponseJson

def render_to(template):
    """
    Shortcut for rendering template with RequestContext.

    If decorated function returns non dict then just return that result
    else use RequestContext for rendering the template.
    """

    def decorator(func):
        def wrapper(request, *args, **kwargs):
            output = func(request, *args, **kwargs)
            if not isinstance(output, dict):
                return output
            else:
                ctx = RequestContext(request)
                return render_to_response(template, output, context_instance=ctx)
        return wrapper
    return decorator


def ajax(func):
    """
    Wrap response of view into JSON format.

    Checks request.method is POST. Return error in JSON in other case.
    """

    def wrapper(request, *args, **kwargs):
        if request.method == 'POST':
            try:
                response = func(request, *args, **kwargs)
            except Exception, ex:
                response = {'error': traceback.format_exc()}
        else:
            response = {'error': {'type': 403, 'message': 'Accepts only POST request'}}
        if isinstance(response, dict):
            return HttpResponseJson(response)
        else:
            return response
    return wrapper



from optparse import make_option
from os.path import dirname, realpath
import os.path
import shutil

from django.core.management.base import BaseCommand, CommandError

COMMON_ROOT = dirname(dirname(dirname(realpath(__file__))))
SKELETON_PATH = os.path.join(COMMON_ROOT, 'data', 'app_template')

class Command(BaseCommand):
    help = 'Create django application skeleton'

    def handle(self, *args, **kwargs):
        if not args:
            raise Exception('Application name required')
        app_name = args[0]
        if os.path.exists(app_name):
            raise Exception('Directory with such name exists')
        copydir(SKELETON_PATH, app_name)


def copydir(src, dst):
    if not os.path.exists(dst):
        print 'Creating directory: %s' % dst
        os.mkdir(dst)
    else:
        print 'Directory already exists: %s' % dst

    for fname in os.listdir(src):
        src_path = os.path.join(src, fname)
        dst_path = os.path.join(dst, fname)
        if os.path.isdir(src_path):
            copydir(src_path, dst_path)
        else:
            print 'Creating file: %s' % dst_path
            shutil.copy(src_path, dst_path)



from BeautifulSoup import BeautifulSoup

from django.template.defaultfilters import urlize as django_urlize

def urlize(data):
    """
    Urlize plain text links in the HTML contents.

    Do not urlize content of A and CODE tags.
    """

    soup = BeautifulSoup(data)
    for chunk in soup.findAll(text=True):
        islink = False
        ptr = chunk.parent
        while ptr.parent:
            if ptr.name == 'a' or ptr.name == 'code':
                islink = True
                break
            ptr = ptr.parent

        if not islink:
            chunk = chunk.replaceWith(django_urlize(unicode(chunk)))

    return unicode(soup)



from django.core.paginator import Paginator, EmptyPage, InvalidPage

from common.templatetags.common_tags import alter_qs


def paginate(qs, request, per_page=15, frame_size=10):
    try:    
        page_number = int(request.GET.get('page', 1))
    except ValueError:
        page_number = 1

    paginator = Paginator(qs, per_page)
    try:    
        page = paginator.page(page_number)
    except (EmptyPage, InvalidPage):
        page_number = 1
        page = paginator.page(1)
    query_string = request.META['QUERY_STRING']

    if page.has_previous():
        page.previous_page_url = alter_qs(query_string, 'page', page.previous_page_number())
    else:
        page.previous_page_url = None

    if page.has_next():
        page.next_page_url = alter_qs(query_string, 'page', page.next_page_number())
    else:
        page.next_page_url = None

    page.first_page_url = alter_qs(query_string, 'page', 1)
    page.last_page_url = alter_qs(query_string, 'page', page.paginator.num_pages)

    urls = []
    if frame_size is None:
        for x in page.paginator.page_range:
            urls.append((x, alter_qs(query_string, 'page', x)))
    else:
        half = int(frame_size / 2.0)
        start = max(1, page.number - int(frame_size / 2.0))
        stop = min(page.paginator.num_pages, page.number + (frame_size - half))
        if start > 1:
            urls.append((None, None))
        for x in xrange(start, stop + 1):
            urls.append((x, alter_qs(query_string, 'page', x)))
        if stop < page.paginator.num_pages:
            urls.append((None, None))
    page.paginator.page_range_urls = urls

    return page



# -*- coding: utf-8 -*-
from django import forms

def build_form(Form, _request, *args, **kwargs):
    """
    Shorcut for building the form instance of given class.
    """

    if 'POST' == _request.method:
        form = Form(_request.POST, _request.FILES, *args, **kwargs)
    else:
        form = Form(*args, **kwargs)
    return form


class DeleteForm(forms.Form):
    """
    Useful for confirmation page on which you ask user if he is
    sure he wants to delete some object.

    next field is optional, you can use it to store the URL to which
    user will be redirected after object deletion.
    """

    next = forms.CharField(required=False, widget=forms.HiddenInput)



from urlparse import urlsplit

from django.conf import settings
from django.views.static import serve

PREFIX = urlsplit(settings.MEDIA_URL).path

class StaticFilesMiddleware(object):
    """
    Django middleware for serving static files instead of using urls.py
    """

    def process_request(self, request):
        if settings.DEBUG:
            if request.path.startswith(PREFIX):
                path = request.path[len(PREFIX):]
                return serve(request, path, settings.MEDIA_ROOT)



from hashlib import sha1

from django.core.cache import cache
from django.utils.encoding import smart_str

def cached(key=None, timeout=300):
    """
    Cache the result of function call.

    Args:
        key: the key with which value will be saved. If key is None
            then it is calculated automatically
        timeout: number of seconds after which the cached value would be purged.
    """
    _key = key

    def func_wrapper(func):
        def args_wrapper(*args, **kwargs):
            # this is workaround of strange python behaviour
            key = _key
            if key is None:
                # Not sure that this will work correct in all cases
                key = sha1(str(func.__module__) + str(func.__name__) +\
                           smart_str(args) +\
                           smart_str(frozenset(kwargs.items()))).hexdigest()
            value = cache.get(key)
            if value:
                return value
            else:
                value = func(*args, **kwargs)
                cache.set(key, value)
                return value
        return args_wrapper
    return func_wrapper
